package asd.graphe;

/**
 * Une classe pour les listes d'adjacences. Une liste d'adjacence est une
 * liste d'entiers, un entier (de type int) reprsentant un sommet. 
 * Quand elle n'est pas mentionne, la complexit est en O(1).
 * (voir le <a href="../../../html/Adjacents.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 3.0 (26/09/02)
 */
public class Adjacents {

    /**
     * Une classe locale pour les listes de sommets (des int).
     */
    private class Chainage {

        int premier;    // la premier entier du chainage
        Chainage suite; // la suite du chainage

        /**
         * Construit une cellule de chainage avec x et c.
         */
        Chainage(int x, Chainage c) {
            premier = x;
            suite = c;
        }
    }

    private Chainage debut;  // le dbut de la liste de sommets

    /**
     * Construit une liste vide.
     */
    public Adjacents() {
	debut = null;
    }

    /**
     * Teste si la liste est non vide.
     */
    public boolean nonVide() {
	return debut != null;
    }

    /**
     * Ajoute une valeur au dbut de la liste.
     */
    public void ajouter(int x) {
        debut = new Chainage(x,debut);
    }

    /**
     * Supprime la valeur x dans la liste.
     * Prcondition : x est dans la liste.
     * Complexit : O(n) o n est la longueur de la liste.
     */
    public void supprimer(int x) {
        if ( debut.premier == x )
            debut = debut.suite;
        else {
            Chainage tmp = debut;
            while ( tmp.suite.premier != x )
                tmp = tmp.suite;
            tmp.suite = tmp.suite.suite;
        }
    }

    /**
     * Retourne et supprime la premire valeur de la liste.
     */
    public int suivant() {
        int x = debut.premier;
	debut = debut.suite;
        return x;
    }

    /**
     * Retourne une copie au premier niveau de la liste.
     */
    public Object clone() {
	Adjacents clone = new Adjacents();
	clone.debut = debut;
	return clone;
    }
}
