package asd.graphe;

/**
 * Une classe pour grer les ensemble de sommets fabriqus par l'algorithme de
 * Kruskal. La runion des n singletons s'effectue en O(n.lg(n)).
 * (voir le <a href="../../../html/Composante.html">source</a>).
 * @author  2005 Marc Gatano
 * @version 3.0 (26/09/02)
 */
public class Composante {

    private Liste[] C; // C[i] = la liste contenant les sommets de la
                       // composante  laquelle appartient le sommet i

    /**
     * Construit un objet Composante pour partionner n sommets.
     * Postrelation : tous les sommets sont isols (i.e. dans un ensemble
     *  un lment).
     * Complexit : O(n).
     */
      public Composante(int n) {
        C = new Liste[n];
        for ( int i = 0; i < n; i++ ) {
            C[i] = new Liste(i);
        }
    }

    /**
     * Teste si les sommets i et j sont dans des  ensembles diffrents.
     * Complexit : O(1).
     */
    public boolean separes(int i, int j) {
        return C[i] != C[j];
    }

    /**
     * Runit les deux ensembles contenant respectivement les sommets
     * i et j.
     * Complexit : O(min(C(i),C(j))) o C(k) = le cardinal de l'ensemble de
     * sommets o se trouve k.
     */
    public void reunir(int i, int j) {
        if ( C[i].longueur > C[j].longueur ) {
            int tmp = i; i = j; j = tmp;
        }
        // on insre les lments de la liste C[i] dans la liste C[j]
        C[j].longueur += C[i].longueur;
        Suite r = C[j].sommets.suivant;
        C[j].sommets.suivant = C[i].sommets;
        Suite s = C[i].sommets;
        while ( s.suivant != null ) {
            C[s.sommet] = C[j];
            s = s.suivant;
        }
        C[s.sommet] = C[j];
        s.suivant = r;
    }

    /**
     * Une classe pour les listes de sommets : une liste de sommet est la
     * longueur de la liste et la suite des sommets.
     */
    private class Liste {

        int longueur;
        Suite sommets;

        /**
         * Fabrique une liste rduite au seul sommet i.
         */
        Liste(int i) {
            sommets = new Suite(i);
            longueur = 1;
        }
    }

    /**
     * Une classe pour les suites de sommets.
     */
    private class Suite {

        int sommet;
        Suite suivant;

        /**
         * Fabrique la suite rduite au seul sommet i.
         */
        Suite (int i) {
            sommet = i;
        }
    }
}
