package asd.graphe;

/**
 * Une classe pour les graphes non orients reprsents  l'aide de listes
 * d'adjacence. Sauf mention contraire, toutes les mthodes sont en O(1).
 * (voir le <a href="../../../html/GrapheNonOriente.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 2.0 (02/12/02)
 */
public class GrapheNonOriente implements Graphe {

    private int ordre;       // le nombre de sommets du graphe
    private int aretes;      // le nombre d'artes du graphe
    private Adjacents[] adj; // les listes des sommets adjacents

    /**
     * Construit un graphe non orient d'ordre n et sans arte.
     * Complexit : O(n).
     */
    public GrapheNonOriente(int n) {
        ordre = n;
        aretes = 0;
        adj = new Adjacents[ordre];

        for ( int i = 0; i < adj.length; i++ ) {
            adj[i] = new Adjacents();
        }
    }

    /**
     * Retourne le nombre de sommets du graphe.
     */
    public int ordre() {
        return ordre;
    }

    /**
     * Retourne le nombre d'artes du graphe.
     */
    public int aretes() {
        return aretes;
    }

    /**
     * Ajoute l'arte (i,j) au graphe.
     * Prcondition : il n'existe pas d'arte joignant i et j.
     */
    public void connecter(int i, int j) {
        adj[i].ajouter(j);
        adj[j].ajouter(i);
        aretes++;
    }

    /**
     * Supprime l'arte (i,j) du graphe.
     * Prcondition : l'arte (i,j) existe.
     * Complexit : O(n) o n est l'ordre du graphe.
     */
    public void deconnecter(int i, int j) {
        adj[i].supprimer(j);
        adj[j].supprimer(i);
        aretes--;
    }

    /**
     * Retourne l'ensemble des sommets adjacents du sommet i.
     */
    public Adjacents adjacents(int i) {
        return (Adjacents) adj[i].clone();
    }
}
