package asd.graphe;

/**
 * Une classe pour les graphes non orients valus.
 * Sauf mention contraire, toutes les mthodes sont en O(1).
 * (voir le <a href="../../../html/GrapheNonOrienteValue.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 3.0 (26/09/02)
 */
public class GrapheNonOrienteValue extends GrapheNonOriente {

    private int[][] V; // V[i][j] est la valuation de l'arte (i,j)
    private static int d = Integer.MAX_VALUE; // la valuation par dfaut

    /**
     * Construit un graphe non orient valu d'ordre n.
     * Complexit : O(n).
     */
    public GrapheNonOrienteValue(int n) {
        super(n);
        V = new int[n][n];
    }

    /**
     * Construit un graphe non orient valu d'ordre n avec d pour
     * valuation par dfaut.
     * Complexit : O(n).
     */
    public GrapheNonOrienteValue(int n, int d) {
        super(n);
        V = new int[n][n];
        this.d = d;
    }
        
    /**
     * Cre une arte joignant les sommets i et j avec la
     * valuation v.
     * Prcondition : il n'existe pas d'arte joignant i et j.
     */
    public void connecter(int i, int j, int v) {
        super.connecter(i,j);
        V[i][j] = v;
        V[j][i] = v;
    }
        
    /**
     * Cre une arte joignant les sommets i et j avec la
     * valuation apr dfaut.
     * Prcondition : il n'existe pas d'arte joignant i et j.
     */
    public void connecter(int i, int j) {
        super.connecter(i,j);
        V[i][j] = d;
        V[j][i] = d;
    }

    /**
     * Retourne la valuation de l'arte (i,j)
     */
    public int valuation(int i, int j) {
        return V[i][j];
    }
}
