package asd.graphe;

/**
 * Une classe pour le parcours en profondeur d'un graphe orient.
 * (voir le <a href="../../../html/PPOriente.html">source</a>).
 * @author  2002 Marc Gatano
 * @version 3.0 (26/09/02)
 */
public class PPOriente {

    private GrapheOriente G;
    
    private int[] d; // la date de dbut de traitement des sommets
    private int[] f; // la date de fin de traitement des sommets
    private int[] p; // p[j] = i <==> i est le pre de j
    private int[] c; // la couleur des sommets
    private int t;   // le temps

    private static final int blanc = 0;
    private static final int gris  = 1;
    private static final int noir  = 2;

    public PPOriente(GrapheOriente G) {
	this.G = G;
	this.d = new int[G.ordre()];
	this.f = new int[G.ordre()];
	this.p = new int[G.ordre()];
	this.c = new int[G.ordre()];
    }

    /**
     * Effectue un parcours en profondeur du graphe et affiche le type de
     * tous les arcs.
     * Complexit : O(S + A) o S est l'ordre du graphe et A le nombre d'arcs.
     */
    public void parcourir() {
	t = 1;
	
	for ( int i = 0; i < G.ordre(); i++ ) {
	    c[i] = blanc;
	    p[i] = -1;
	}
        // tous les sommets sont blancs et aucun sommet n'a de pre

	for ( int i = 0; i < G.ordre(); i++ ) {
	    if ( c[i] == blanc ) {
		visiter(i);
	    }
	}
    }

    /**
     * Visite le sous-graphe de sommet i en profondeur et affiche le type de
     * tous les arcs.
     * Prcondition : i est blanc.
     * Postrelation : i est noir et le type de tous les arcs accessibles depuis
     * i a t affich.
     * Complexit : O(Ai) o Ai est le nombre d'arcs accessibles depuis i.
     */
    private void visiter(int i) {
	c[i] = gris;
	d[i] = t++;
	Adjacents adj = G.adjacents(i);
	
	while ( adj.nonVide() ) {
	    int s = adj.suivant();

            // dbut de tratement
	    switch ( c[s] ) {
            case blanc :
                afficher(i,s,"de liaison");
		p[s] = i;
		visiter(s);
                break;
            case gris :
                afficher(i,s,"retour");
                break;
            case noir :
                afficher(i,s,"crois/couvrant");
            }
            // fin de tratement
	}
	c[i] = noir;
	f[i] = t++;
    }

    // affiche l'arc (i,j) et son type. la chane s
    private void afficher(int i, int j, String s) {
        System.out.println(i + " --> " + j + " est un arc " + s);
    }
}
