package asd.graphe;

import asd.base.ObjetAClef;
import asd.tri.TasCroissant;

/**
 * Une classe pour l'algorithme de Prim, pour des graphes non orients
 * connexes, valus.
 * (voir le <a href="../../../html/Prim.html">source</a>).
 * @author  2005 Marc Gatano
 * @version 3.0 (18/04/05)
 */
public class Prim {

    private GrapheNonOrienteValue G; // le graphe non orient connexe valu

    public Prim(GrapheNonOrienteValue G) {
	this.G = G;
    }

    /**
     * Retourne un arbre couvrant de poids minimum du graphe G.
     * Complexit : O(A.lg(A)) o A est le nombre d'artes de G.
     */
    public GrapheNonOrienteValue acm() {

        TasCroissant T = new TasCroissant(G.aretes());
        GrapheNonOrienteValue Ge = new GrapheNonOrienteValue(G.ordre());
        boolean[] dansSprime = new boolean[G.ordre()];

        dansSprime[0] = true;           // dansSprime[u] <==> u est marqu
        int m = 1;                   // le nombre de sommets marqus

        Adjacents adj = G.adjacents(0); // on initialise T
        while ( adj.nonVide() ) {
            int s = adj.suivant();
            T.ajouter(new ObjetAClef(new Arete(0,s),G.valuation(0,s)));
        }

        while ( m < G.ordre() ) {   
            // si m < G.ordre() alors T n'est pas vide (point 2 de l'invariant).
            Arete uv = (Arete) T.extraireMinimum().info(); 
            int u = uv.origine();
            int v = uv.extremite();
            // uv = (u,v) est dans Ge et u est marqu
	    if ( ! dansSprime[v] ) {
                // on peut connecter (u,v) dans Ge
                Ge.connecter(u,v,G.valuation(u,v));
		dansSprime[v] = true;
                m++;
		adj = G.adjacents(v);
		while ( adj.nonVide() ) {
		    int s = adj.suivant();
		    if ( ! dansSprime[s] ) {
			T.ajouter(new ObjetAClef(new Arete(v,s),G.valuation(v,s)));
		    }
		}
	    }
        }
        // INVARIANT :
        // 0) u est dans S' <==> dansSprime[u] est vrai
	// 1) Ge = (S',E) est un ACM du sous-graphe de G engendr par S', avec
        //    |S'| = m
	// 2) u est dans S' avec (u,v) arte de G et v non marqu
        //    ==> (u,v) est dans T
        // VARIANT :
        //   le couple ( m, - T.taille() ) crot lexicographiquement et
        //   est born par ( G.ordre(), 0 )
        //
	return Ge;
    }
}
